# ***************************************************************************
# Copyright 2017-2018 VMware, Inc.  All rights reserved. VMware Confidential.
# ***************************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# ***************************************************************************
import json
import os
import nsx_logging
from cli.utils.command_utils import invoke_command
from cli.utils.constants import NODE_TYPE_EDGE
from cli.utils.constants import NODE_TYPE_AUTONOMOUS_EDGE
from cli.utils.constants import NODE_TYPE_HOST_CENTOSKVM
from cli.utils.constants import NODE_TYPE_HOST_SLESKVM
from cli.utils.constants import NODE_TYPE_HOST_ESX
from cli.utils.constants import NODE_TYPE_HOST_RHELKVM
from cli.utils.constants import NODE_TYPE_HOST_UBUNTUKVM
from cli.utils.constants import NODE_TYPE_PUBLIC_CLOUD_GATEWAY
from cli.utils.constants import NODE_TYPE_BAREMETAL_UBUNTU_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_RHEL_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_CENTOS_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_SLES_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_OEL_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_WINDOWS_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_RHEL_CONTAINER
from cli.utils.constants import NODE_TYPE_BAREMETAL_CENTOS_CONTAINER
from cli.utils.node_utils import is_windows
from cli.utils.location_utils import FilePaths
from cli.utils.node_utils import get_node_type
from cli.utils.error_code import ErrorCode as ec

lg = nsx_logging.getLogger("cli.commands.lcp_common.nsx_proxy.client")

_NSXPROXY_CLI = {
    NODE_TYPE_EDGE: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-proxy/nsx-proxy-cli"
        },
    NODE_TYPE_PUBLIC_CLOUD_GATEWAY: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-proxy/nsx-proxy-cli"
        },
    NODE_TYPE_HOST_SLESKVM: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-proxy/nsx-proxy-cli"
        },
    NODE_TYPE_HOST_CENTOSKVM: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-proxy/nsx-proxy-cli"
        },
    NODE_TYPE_HOST_ESX: {
        'bin': "/opt/vmware/nsx-cli/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-proxy/nsx-proxy-cli"
        },
    NODE_TYPE_HOST_RHELKVM: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-proxy/nsx-proxy-cli"
        },
    NODE_TYPE_HOST_UBUNTUKVM: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-proxy/nsx-proxy-cli"
        },
    NODE_TYPE_AUTONOMOUS_EDGE: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-proxy/nsx-proxy-cli"
        },
    NODE_TYPE_BAREMETAL_UBUNTU_SERVER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-proxy/nsx-proxy-cli"
        },
    NODE_TYPE_BAREMETAL_RHEL_SERVER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-proxy/nsx-proxy-cli"
        },
    NODE_TYPE_BAREMETAL_CENTOS_SERVER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-proxy/nsx-proxy-cli"
        },
    NODE_TYPE_BAREMETAL_OEL_SERVER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-proxy/nsx-proxy-cli"
        },
    NODE_TYPE_BAREMETAL_SLES_SERVER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-proxy/nsx-proxy-cli"
        },
    NODE_TYPE_BAREMETAL_RHEL_CONTAINER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-proxy/nsx-proxy-cli"
        },
    NODE_TYPE_BAREMETAL_CENTOS_CONTAINER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-proxy/nsx-proxy-cli"
        },
    }

if is_windows():
    _NSXPROXY_CLI.update({
        NODE_TYPE_BAREMETAL_WINDOWS_SERVER: {
            'bin': os.path.join(FilePaths.NSX_ROOT, "nsx-cli", "cli", "bin", "nsx-appctl"),
            'socket': os.path.join(FilePaths.NSX_ROOT, "nsx-proxy", "nsx-proxy-cli")
        }
    })


def has_nsxproxy_cli(node_type):
    return node_type in (NODE_TYPE_AUTONOMOUS_EDGE,
                         NODE_TYPE_BAREMETAL_CENTOS_CONTAINER,
                         NODE_TYPE_BAREMETAL_CENTOS_SERVER,
                         NODE_TYPE_BAREMETAL_RHEL_CONTAINER,
                         NODE_TYPE_BAREMETAL_RHEL_SERVER,
                         NODE_TYPE_BAREMETAL_UBUNTU_SERVER,
                         NODE_TYPE_BAREMETAL_SLES_SERVER,
                         NODE_TYPE_BAREMETAL_OEL_SERVER,
                         NODE_TYPE_BAREMETAL_WINDOWS_SERVER,
                         NODE_TYPE_EDGE,
                         NODE_TYPE_HOST_CENTOSKVM,
                         NODE_TYPE_HOST_SLESKVM,
                         NODE_TYPE_HOST_ESX,
                         NODE_TYPE_HOST_RHELKVM,
                         NODE_TYPE_HOST_UBUNTUKVM,
                         NODE_TYPE_PUBLIC_CLOUD_GATEWAY)


def invoke_nsxproxy_appctl_command(cmd_list):
    node_type = get_node_type()
    if has_nsxproxy_cli(node_type):
        cmd = [_NSXPROXY_CLI[node_type]['bin'], "-t",
               _NSXPROXY_CLI[node_type]['socket']]
        cmd.extend(cmd_list)
        rc, out, err = invoke_command(cmd)
        return (rc, out, err)
    else:
        lg.error("Node type: %s does not support running"
                 " the nsxproxy CLI command: %s", node_type, cmd_list)
        return (-1, None, "invalid command")


def get_nsxproxy_cli_output(command, argv=None):
    if argv is None:
        argv = []
    cmd_list = [command]

    length = len(argv)
    if length  > 0:
        cmd_list.extend(argv)
    rc, out, err = invoke_nsxproxy_appctl_command(cmd_list)
    if rc != 0:
        lg.error("Failed to invoke nsxproxy CLI command, rc: %d, err: %s",
                 rc, err, error_code=ec.OPERATION_FAILED)
        out = None

    return out


def deserialize_to_obj(json_txt):
    try:
        if json_txt is None:
            return None
        output = json.loads(json_txt)
    except (TypeError, ValueError) as err:
        lg.error("Failed to deserialize json text: "
                 "%s to object. Error: %s" % (json_text, str(err)))
        return None
    return output
