# Copyright (C) 2020 VMware, Inc. All rights reserved.
# -- VMware Confidential

import queue
import threading
import traceback
import sha.config
from sha.contrib.monitor.abc_base import Monitor
from sha.contrib.monitor.traceCollector.internal.connection import Connection

class IntTraceMonitor(Monitor):
    """
    IntTraceMonitor waits for the span data
    """

    def __init__(self):
        self._logger = sha.config.get_logger(__name__)
        self._stop = None
        self._queue = queue.Queue(500)
        self._msg_handle_thread = threading.Thread(target=self._handle_notif, daemon=True)
        self._msg_recv_thread = threading.Thread(target=self._recv_spans, daemon=True)

    def start(self):
        self._stop = False
        Connection().start(self._queue)
        self._msg_handle_thread.start()
        self._msg_recv_thread.start()

    def _recv_spans(self):
        if not Connection.is_connected:
            return
        try:
            Connection.wait_for_data()
        except Exception as e:
            trace = traceback.format_exc()
            self._logger.warning("TM:socket is closead, Exception:{}, trace:{}".format(e, trace))

    def _handle_notif(self):
        while not self._stop:
            try:
                data = self._queue.get(timeout=1)
                self._logger.debug("TM:notify data is {}".format(data))
                self.notify(data)
                self._queue.task_done()
            except queue.Empty:
                continue
            except Exception as e:
                trace = traceback.format_exc()
                self._logger.error("Process trace notification failed, Exception:{}, trace:{}".format(e, trace))

    def wait_to_stop(self):
        self._msg_handle_thread.join()
        self._msg_recv_thread.join()

    def stop(self):
        self._stop = True
        Connection.close()
